//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Color notehead plugin version 1.2
//	Noteheads are colored according to pitch, according to the BoomWhackers convention. 
//  Each pitch has a different color. C and C# have a different color. C# and Db have the same color.
//  User can change to color by modifying the colors array. First element is C, second C# etc...
//
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//
// History:
//
// 1.2  Dez 2014 by Djamana
//  * reordered colors so related/consonant tones have simulare color
//    Practical print and readablity on paper not tested so far
//
// 1.1  Jan 2013 by Djamana
//  * adjusted colors to be more printerfriendly 
//    (i.e. to contrast of a light yellow note on white sheet of paper goes to sth like nothing)
//    Well 'yellow' is still a catastoph 
//  * Recalling the plugin makes Notes black again
//  * added more source comments and this History


//
// 1.0 xxx 2008 by Schweer
//  * inital version

// Rainbow scale applied to circle of fifths (started on C with magenta)
// 
// http://virtuosoism.com/Color-Sound%20Relationships.pdf 
// (Combined with with http://www.color-picker.de)


var colors = [

	/*C*/   new QColor(226, 48,140), new QColor(  0,175,136),
	/*D*/   new QColor(243,111, 45), new QColor(  0,133,202),
  /*E*/   new QColor(247,232, 41), 
	/*F*/   new QColor(139, 75,157), new QColor(105,190, 87),
	/*G*/   new QColor(240, 65, 72), new QColor(  0,176,200),
	/*A*/   new QColor(251,169, 37), new QColor( 68, 98,174),
	/*B*/   new QColor(198,217, 55)
];

// old Version 1.1 colormap - Rainbow colors on cromatic scale
//var colors = [
//	/*C*/ new QColor(226, 28,72), new QColor(242,102, 34),
//	
//	/*D*/ new QColor(249,157,28), new QColor(255,204, 51),
//	
//    /*E   new QColor(255,243,43),*/ 
//    /*E*/ new QColor(238,207,  0), 
//	
//	/*F   new QColor(188,216, 95),new QColor( 98,188, 71),*/
//	/*F*/ new QColor(155,176, 77),new QColor( 98,188, 71),
//	
//	/*G*/ new QColor(  0,156,149),new QColor(  0,113,187),
//	
//	/*A*/ new QColor( 94, 80,161),new QColor(141, 91,166),
//	
//	/*B*/ new QColor(207, 62,150)
//];
// http://boomwhackers.com/wp-content/uploads/2011/Color%20Signals.pdf
// C	#e31a47	227	26	 71 
// D	#f89c1c	248	156	 28
// E	#fff229	255	242	 41
// F	#d4e05a	212	224	 90
// G	#009c95	  0 156	149
// A	#5d4fa1	 93	 79	161
// B	#ce3895	206	 56	149

//------------------------------------------------------

// Standardcolor is black
var COLOR_BLACK = new QColor(0,0,0);

var NON_COLORED = COLOR_BLACK;


//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init()
      {
      // print("test script init");
      }

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
      {
			
      var cursor = new Cursor(curScore);
	  
	  // Do for all Staves/Tracks ...
      for (var staff = 0; staff < curScore.staves; staff++) {
            cursor.staff = staff;
			
			// Do for all 4 voices ...
            for (var voice = 0; voice <= 3; voice++) {
                  cursor.voice = voice;
				  
				  //Do for all chords (/notes) ...
                  cursor.rewind();  // set cursor to first chord/rest
                  while (!cursor.eos()) {
				  
                        if (cursor.isChord()) {
                              var chord = cursor.chord();
							  
							  //Do for all notes ...
							  for (var i = 0; i < chord.notes; i++) {
							       var note = chord.note(i)
								   
							       note.color = getColor( note.color,
								                          note.pitch
														 );
							   }
                        
						}
						
						// get next chord (/notes)	  
                        cursor.next();
                        }
                  }
            }
 	  }
//////////////////////////////////////////////
/// getColor - returns color according to pitch
//
function getColor( note_color, note_pitch ) {

	if ( isColoredNote(note_color) ) {
		return( COLOR_BLACK ) ;
	
	} else {
	    var colorIndex = note_pitch % 12;
		return( colors[ colorIndex ] );

	}

}

////////////////////////////////////////////////////////////////////
/// isColoredNote - returns true if first passed note is not 'black'
//
var pIsColoredNote = undefined;
function isColoredNote(note_color) {
	// only test the first note and cache the result 
	// to apply the result to all following requests
	if (typeof pIsColoredNote === 'undefined') {
		pIsColoredNote = (NON_COLORED != note_color);
	}
		
	return( pIsColoredNote );
}
	  
//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Color/Uncolor Notes (Ver 1.2)',
      init: init,
      run:  run
      };

mscorePlugin;

